package com.datacloudsec.utils;

import com.datacloudsec.utils.BASE64Util;
import java.security.Key;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;
import java.util.HashMap;
import java.util.Map;
import javax.crypto.Cipher;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

public class RSAUtil {

   private static final String ALGORITHM_RSA = "RSA";
   private static final String ALGORITHM_SHA1PRNG = "SHA1PRNG";
   private static final int KEY_SIZE = 1024;
   private static final String PUBLIC_KEY = "RSAPublicKey";
   private static final String PRIVATE_KEY = "RSAPrivateKey";
   private static final String TRANSFORMATION = "RSA/None/NoPadding";


   public static String decryptByPrivateKey(String data, String key) throws Exception {
      return new String(decryptByPrivateKey((byte[])BASE64Util.decodeToByte(data), key));
   }

   public static String encryptByPublicKey(String data, String key) throws Exception {
      return encryptByBytePublicKey(data.getBytes(), key);
   }

   public static String encryptByBytePublicKey(byte[] data, String key) throws Exception {
      return BASE64Util.encodeByte(encryptByPublicKey((byte[])data, key));
   }

   public static byte[] decryptByPrivateKey(byte[] data, String key) throws Exception {
      byte[] keyBytes = BASE64Util.decodeToByte(key);
      PKCS8EncodedKeySpec pkcs8KeySpec = new PKCS8EncodedKeySpec(keyBytes);
      KeyFactory keyFactory = KeyFactory.getInstance("RSA");
      PrivateKey privateKey = keyFactory.generatePrivate(pkcs8KeySpec);
      Cipher cipher = Cipher.getInstance("RSA/None/NoPadding", new BouncyCastleProvider());
      cipher.init(2, privateKey);
      return cipher.doFinal(data);
   }

   public static byte[] encryptByPublicKey(byte[] data, String key) throws Exception {
      byte[] keyBytes = BASE64Util.decodeToByte(key);
      X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(keyBytes);
      KeyFactory keyFactory = KeyFactory.getInstance("RSA");
      PublicKey publicKey = keyFactory.generatePublic(x509KeySpec);
      Cipher cipher = Cipher.getInstance("RSA/None/NoPadding", new BouncyCastleProvider());
      cipher.init(1, publicKey);
      return cipher.doFinal(data);
   }

   public static String getPrivateKey(Map keyMap) {
      Key key = (Key)keyMap.get("RSAPrivateKey");
      return BASE64Util.encodeByte(key.getEncoded());
   }

   public static String getPublicKey(Map keyMap) {
      Key key = (Key)keyMap.get("RSAPublicKey");
      return BASE64Util.encodeByte(key.getEncoded());
   }

   public static Map initKey(String seed) throws Exception {
      KeyPairGenerator keyPairGen = KeyPairGenerator.getInstance("RSA");
      SecureRandom random = SecureRandom.getInstance("SHA1PRNG");
      random.setSeed(seed.getBytes());
      keyPairGen.initialize(1024, random);
      KeyPair keyPair = keyPairGen.generateKeyPair();
      RSAPublicKey publicKey = (RSAPublicKey)keyPair.getPublic();
      RSAPrivateKey privateKey = (RSAPrivateKey)keyPair.getPrivate();
      HashMap keyMap = new HashMap(2);
      keyMap.put("RSAPublicKey", publicKey);
      keyMap.put("RSAPrivateKey", privateKey);
      return keyMap;
   }

   public static void main(String[] args) throws Exception {
      String source = "123456";
      String seed = "datacloudsec_asy_!@#";
      System.out.println("原文：\n" + source);
      Map keyMap = initKey(seed);
      String publicKey = getPublicKey(keyMap);
      String privateKey = getPrivateKey(keyMap);
      System.out.println("公钥：\n" + publicKey);
      System.out.println("私钥：\n" + privateKey);
      String encodedStr = encryptByPublicKey((String)source, publicKey);
      System.out.println("密文：\n" + encodedStr);
      String decodedStr = decryptByPrivateKey((String)encodedStr, privateKey);
      System.out.println("解密后的结果：\n" + decodedStr);
   }
}
